--[[
SIMPLE THREAT METER
Release: 16th August 2008
Author: Stormknight/Amethyst

Description:
When in combat, shows two vertical bars. The thin red bar indicates the threat level of the current target.
By design, this is always treated as 100% of current threat.
The larger vertical bar shows your threat on the target compared to the other bar, on a sliding scale from 0 to 130%.

This is the initial release of this mod and it is, as it's reason to exist, been designed to be simple.
As such, I am wary of adding too much stuff into it, though simple in execution can still allow for comprehensive
set of options to decide how it should behave.
The TO DO list:
* oRA2 integration, to do the following:
* * * Show the MT as the reference tank bar rather than the current target.
* * * Show multiple MT reference bars. Useful when you have to stay below both tanks on Twin Eredar etc
]]--
SimpleThreatMeter = LibStub("AceAddon-3.0"):NewAddon("SimpleThreatMeter", "AceEvent-3.0", "LibBars-1.0", "AceConsole-3.0")
local bars = LibStub("LibBars-1.0")
local SharedMedia = LibStub("LibSharedMedia-3.0")
local mod = SimpleThreatMeter
local PlayerName = UnitName("player")
local Threat = LibStub("Threat-2.0")

--
-- OPTIONS SECTION
--

function mod:GetLocked(info)
	return self.db.profile.locked
end

function mod:SetLocked(info,newValue)
	self.db.profile.locked = newValue
	if (self.db.profile.locked) then
		self.threatBarSet:HideAnchor()
	else
		self.threatBarSet:ShowAnchor()
	end
end

function mod:GetHeight(info)
	return self.db.profile.height
end

function mod:SetHeight(info,newValue)
	self.db.profile.height = newValue
	self.threatBar:SetHeight(self.db.profile.height)
	self.tankBar:SetHeight(self.db.profile.height)
	self.threatBarSet:SetHeight(self.db.profile.height)
end

function mod:GetWidth(info)
	return self.db.profile.width
end

function mod:SetWidth(info,newValue)
	self.db.profile.width = newValue
	self.threatBar:SetWidth(self.db.profile.width)
end

function mod:GetFontSize(info)
	return self.db.profile.fontsize
end

function mod:SetFontSize(info,newValue)
	self.db.profile.fontsize = newValue
	self.threatBarSet:SetFont(nil, self.db.profile.fontsize)
end

function mod:GetICfade(info)
	return self.db.profile.icfade
end

function mod:SetICfade(info,newValue)
	self.db.profile.icfade = newValue
	self:SetOverallAlpha(self.db.profile.icfade)
end

function mod:GetTexture(info)
	return self.db.profile.textureValue
end

function mod:SetTexture(info,newValue)
	self.db.profile.textureValue = newValue
	local tmplist = SharedMedia:List('statusbar')
	self.threatBarSet:SetTexture(SharedMedia:Fetch("statusbar", tmplist[newValue]))
end

function mod:ChatCommand(input)
    if not input or input:trim() == "" then
--        LibStub("AceConfigDialog-3.0"):Open("SimpleThreatMeter")
	InterfaceOptionsFrame_OpenToFrame(self.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(SimpleThreatMeter, "wh", "SimpleThreatMeter", input)
    end
end
--[[
function mod:TestFunction()
	self:DebugMessage("oRA version:" .. oRA.version)
	local maintanks = oRA.maintanktable
	for i, name in pairs(maintanks) do
		self:DebugMessage("MT " .. i .. " : " .. name)
	end
end
]]--
local options = {
	name = "SimpleThreatMeter",
	handler = SimpleThreatMeter,
	type = 'group',
	args = {
	    	locked = {
	    		type = "toggle",
	    		name = "Locked",
	    		desc = "Lock or Unlock the mod.",
	    		get = "GetLocked",
	    		set = "SetLocked",
	    		width = "full",
	    		order = 0,
	    	},
	    	testbars = {
	    		type = "execute",
	    		name = "Show Test",
	    		desc = "Display the threat bars in configuration mode.",
	    		func = "ConfigDisplaybars",
	    		order = 10,
	    	},
	    	testbarsgone = {
	    		type = "execute",
	    		name = "Hide Test",
	    		desc = "Remove the test threat bars.",
	    		func = "ConfigRemovebars",
	    		order = 11,
	    	},
	    	sizing = {
	    		type = 'group',
	    		name = "Resizing",
	    		inline = true,
	    		order = 20,
	    		args = {
			    	height = {
			    		type = "range",
			    		name = "Threat Bar Height",
			    		desc = "The height of the threat bars.",
			    		min = 100,
			    		max = 600,
			    		step = 1,
			    		bigStep = 5,
			    		get = "GetHeight",
			    		set = "SetHeight",
	    				width = "full",
			    		order = 1,
			    	},
			    	width = {
			    		type = "range",
			    		name = "Threat Bar Width",
			    		desc = "The width of the threat bars.",
			    		min = 10,
			    		max = 100,
			    		step = 1,
			    		bigStep = 5,
			    		get = "GetWidth",
			    		set = "SetWidth",
	    				width = "full",
			    		order = 2,
			    	},
			    	fontsize = {
			    		type = "range",
			    		name = "Font Size",
			    		desc = "The size of the font to use.",
			    		min = 8,
			    		max = 20,
			    		step = 1,
			    		bigStep = 1,
			    		get = "GetFontSize",
			    		set = "SetFontSize",
	    				width = "full",
			    		order = 3,
			    	},
	    		},
	    	},
	    	fading = {
	    		type = 'group',
	    		name = "Visual",
	    		inline = true,
	    		order = 30,
	    		args = {
	    			texture = {
	    				type = "select",
	    				style = "dropdown",
	    				name = "Bar Texture",
					values = function()
						return_array = SharedMedia:List('statusbar')
						return return_array
					end,
			    		get = "GetTexture",
			    		set = "SetTexture",
	    				order = 0,
	    			},
	    			icfade = {
		    			type = "range",
		    			name = "Transparency",
		    			desc = "How solid you want the threat bar to be. 100% is solid and 10% is almost invisible.",
		    			min = 0.1,
		    			max = 1,
		    			step = 0.1,
		    			bigStep = 0.1,
		    			isPercent = true,
		    			get = "GetICfade",
		    			set = "SetICfade",
	    				order = 1,
	    			},
	    		},
	    	},
--[[
	    	testfunc = {
	    		type = "execute",
	    		name = "DEBUG TEST",
	    		desc = "DEBUG TEST FUNCTION",
	    		func = "TestFunction",
	    		order = 100,
	    	},
]]--
	},
}

local defaults = {
	profile = {
		locked = true,
		x = 400,
		y = 200,
		height = 200,
		width = 40,
		fontsize = 10,
		icfade = 0.8,
		texture = "Smooth v2",
	},
}

--
-- MAIN MOD SECTION
--


function mod:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SimpleThreatMeterDB", defaults, "Default")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SimpleThreatMeter", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SimpleThreatMeter", "SimpleThreatMeter")
	self:RegisterChatCommand("simplethreatmeter", "ChatCommand")
	self:RegisterChatCommand("stm", "ChatCommand")

	self.threatBarSet = self:NewBarGroup("Vert Test", bars.BOTTOM_TO_TOP, self.db.profile.height, self.db.profile.width)
	self.threatBarSet:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", self.db.profile.x, self.db.profile.y)

	local tmplist = SharedMedia:List('statusbar')
	self.threatBarSet:SetTexture(SharedMedia:Fetch("statusbar", tmplist[self.db.profile.textureValue]))

	self.threatBarSet.RegisterCallback(self, "AnchorClicked")
	self.threatBarSet.RegisterCallback(self, "AnchorMoved")

	self.threatBarSet:SetColorAt(1.00, 1.0, 0.0, 0.0, 1.0)
	self.threatBarSet:SetColorAt(0.70, 1.0, 0.0, 0.0, 1.0)
	self.threatBarSet:SetColorAt(0.40, 0.4, 1.0, 0.0, 1.0)
	self.threatBarSet:SetColorAt(0.00, 0.4, 1.0, 0.0, 1.0)
	self.threatBarSet:SetFont(nil, self.db.profile.fontsize)

	if (self.db.profile.locked) then
		self.threatBarSet:HideAnchor()
	else
		self.threatBarSet:ShowAnchor()
	end
end


function mod:OnEnable()
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")

	Threat.RegisterCallback(self, "Activate", "UpdateThreatBar")
	Threat.RegisterCallback(self, "Deactivate", "UpdateThreatBar")
	Threat.RegisterCallback(self, "ThreatUpdated", "UpdateThreatBar")
	Threat:RequestActiveOnSolo(true)
	self:SetOverallAlpha(self.db.profile.icfade)
end

-- Invoked every time we need to make sure that the bars exist.
function mod:MakeThreatBars()
	if ( self.threatBarSet:GetBar("TankThreat") ) then
		-- exists
		-- self:DebugMessage("Bars already exist!")
	else
		-- self:DebugMessage("Created tank bar!")
		self.tankBar = self.threatBarSet:NewCounterBar("TankThreat", "", 0, 130, nil, nil, self.db.profile.height, 10, false)
		self.tankBar:SetWidth(10)
		self.tankBar:SetHeight(self.db.profile.height)
		self.tankBar:SetValue(100)
	end
	if ( self.threatBarSet:GetBar("TargetThreat") ) then
		-- exists
	else
		self.threatBar = self.threatBarSet:NewCounterBar("TargetThreat", "", 1, 130, nil, nil, self.db.profile.height, self.db.profile.width, false)
		self.threatBar:SetWidth(self.db.profile.width)
		self.threatBar:SetHeight(self.db.profile.height)
		self.threatBar:SetValue(0)
	end
	self:SetOverallAlpha(self.db.profile.icfade)
end

function mod:UpdateThreatBar()
	local playerGUID = UnitGUID("player")
	local targetGUID = UnitGUID("target")
	local tankGUID = UnitGUID("targettarget")
	
	local threatRatio = 1
	local playerThreatVal = 0
	local tankThreatVal = 0	
	
	if ( InCombatLockdown() ) then
		-- Check that the player is targetting a non-friendly mob
		if ( UnitCanAttack("player", "target") ) then
			-- Is the mob targetting the player?
			if ( UnitIsUnit("player","targettarget") ) then
				threatRatio = 100
			else
				playerThreatVal = Threat:GetThreat(playerGUID,targetGUID)
				tankThreatVal =  Threat:GetThreat(tankGUID,targetGUID)
				-- Make sure we're not creating maths errors
				if ((playerThreatVal > 0) and (tankThreatVal > 0)) then
					threatRatio = 100 * (playerThreatVal / tankThreatVal)
				end
				-- Stop ratio from pushing the max size of the bar higher
				if (threatRatio > 130) then
					threatRatio = 130
				end
			end
		end
		self.threatBar:SetValue(threatRatio)
		self.threatBar:SetLabel(("%2.0f%%"):format(threatRatio))
		-- self:DebugMessage("ThreatBar: " .. playerThreatVal .. " / " .. tankThreatVal .. " = " .. threatRatio .. "%")
	else
		-- self:DebugMessage("ThreatBar update but OOC.")
	end
end

function mod:DebugMessage(text)
	DEFAULT_CHAT_FRAME:AddMessage(text, 1, 1, 0);
end

-- ENTER COMBAT
function mod:PLAYER_REGEN_DISABLED()
	-- self:DebugMessage("In Combat!")
	-- Can get MT info from oRA2 if the user has it.
	self:MakeThreatBars()
	self:SetOverallAlpha(self.db.profile.icfade)
end

-- EXIT COMBAT
function mod:PLAYER_REGEN_ENABLED()
	-- self:DebugMessage("Combat Ended!")
	self.threatBar:Fade(0.2)
	self.tankBar:Fade(0.2)
end

function mod:SetOverallAlpha(alphaValue)
	if ( self.tankBar ) then
		self.tankBar:SetAlpha(alphaValue)
		self.threatBar:SetAlpha(alphaValue)
	end
end

function mod:AnchorClicked(cbk, group, button)
	if button == "RightButton" then
		InterfaceOptionsFrame_OpenToFrame(self.optionsFrame)
	end
end

function mod:AnchorMoved(cbk, group, x, y)
	self.db.profile.x = floor(x)
	self.db.profile.y = floor(y) - self.db.profile.height
end

function mod:ConfigDisplaybars()
--	self:DebugMessage("ConfigDisplaybars")
	self:UpdateThreatBar()
	self:MakeThreatBars()
	self:SetOverallAlpha(self.db.profile.icfade)
	self.threatBar:SetValue(50)
end

function mod:ConfigRemovebars()
	self.threatBar:Fade(1)
	self.tankBar:Fade(1)
end